package jp.deadend.noname.ntl;

import android.content.Context;
import android.content.pm.PackageManager;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.Filter;
import android.widget.Filterable;
import android.widget.TextView;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

class AppListAdapter extends BaseAdapter implements Filterable {
	private List<AppNameInfo> mList;
	private List<AppNameInfo> mOriginalValues;
	private Map<String, Drawable> mIcons;
	private int mFontSize;
	private int mIconSize;
	private int mPaddingSize;
	private Context mContext;
	private RomajiConverter mRC;

	AppListAdapter(Context context, List<AppNameInfo> items) {
		mList = items;
		mOriginalValues = new ArrayList<AppNameInfo>(mList);
		mFontSize = PrefsActivity.DEFAULT_FONT_SIZE;
		mIcons = new HashMap<String, Drawable>();
		mContext = context;
		mRC = new RomajiConverter();
	}

	void setFontSize(int size) {
		mFontSize = Math.max(10, Math.min(size, 100));
		mPaddingSize = Math.round(size*0.5f);
		mIconSize = size+mPaddingSize;
	}

	int getFontSize() {
		return mFontSize;
	}

	@Override
	public int getCount() {
		return mList.size();
	}

	@Override
	public AppNameInfo getItem(int position) {
		return mList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	private void setIconDrawable(TextView tv, Drawable icon, int size, int paddingSize) {
		icon.setBounds(0, 0, size, size);
		tv.setCompoundDrawablePadding(paddingSize);
		tv.setCompoundDrawables(icon, null, null, null);
	}

	private class LoadIconTask extends AsyncTask<String, Void, Drawable> {
		private TextView mTextView;
		private String mTag;
		private int mIconSize;
		private int mPaddingSize;

		public LoadIconTask(TextView textView, int size, int padding) {
			mTextView = textView;
			mTag = textView.getTag().toString();
			mIconSize = size;
			mPaddingSize = padding;
		}

		@Override
		protected Drawable doInBackground(String... packageName) {
			try {
				Drawable icon = mContext.getPackageManager().getApplicationIcon(packageName[0]);
				mIcons.put(packageName[0], icon);

				return icon;
			} catch (PackageManager.NameNotFoundException e) {
			}

			return null;
		}

		@Override
		protected void onPostExecute(Drawable result) {
			if (mTag.equals(mTextView.getTag())) {
				if (result != null) setIconDrawable(mTextView, result, mIconSize, mPaddingSize);
			}
		}
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		TextView tv = (TextView)convertView;

		if (tv == null) {
			tv = new TextView(mContext);
		}
		tv.setTextSize(mFontSize);
		AppNameInfo app = mList.get(position);
		tv.setText(app.name);

		tv.setTag(app.packageName);
		Drawable icon = mIcons.get(app.packageName);
		if (icon == null) {
			LoadIconTask task = new LoadIconTask(tv, mIconSize, mPaddingSize);
			task.execute(app.packageName);
		} else {
			setIconDrawable(tv, icon, mIconSize, mPaddingSize);
		}

		return tv;
	}

	@Override
	public Filter getFilter() {
		Filter filter = new Filter() {
			@SuppressWarnings("unchecked")
			@Override
			protected void publishResults(CharSequence constraint, FilterResults results) {
				mList = (List<AppNameInfo>)results.values;
				notifyDataSetChanged();
			}

			@Override
			protected FilterResults performFiltering(CharSequence constraint) {
				FilterResults results = new FilterResults();
				List<AppNameInfo> filteredArrList = new ArrayList<AppNameInfo>();

				if (constraint == null || constraint.length() == 0) {
					results.count = mOriginalValues.size();
					results.values = mOriginalValues;
				} else {
					Pattern p;
					try {
//							p = Pattern.compile(constraint.toString(), Pattern.CASE_INSENSITIVE);
						p = Pattern.compile(mRC.makeRegExp(constraint.toString()), Pattern.CASE_INSENSITIVE);
					} catch (PatternSyntaxException e) {
						results.count = mOriginalValues.size();
						results.values = mOriginalValues;
						return results;
					}
					Matcher m = p.matcher("");
					for (AppNameInfo app : mOriginalValues) {
						m.reset(app.name);
						if (m.find()) {
							filteredArrList.add(app);
						}
					}
					results.count = filteredArrList.size();
					results.values = filteredArrList;
				}

				return results;
			}
		};

		return filter;
	}
}
