package jp.deadend.noname.ntl;

import android.content.Context;
import android.util.Log;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;

class CacheFile {
	private static final String CACHE_FILE = "ntl.cache";
	private static final String EXCLUDE_FILE = "ntl.exclude";
	private static final String RECENT_FILE = "ntl.recent";

	private static List<AppNameInfo> loadAppNameInfoList(Context context, String fileName) {
		FileInputStream in;
		try {
			in = context.openFileInput(fileName);
		} catch (FileNotFoundException e) {
			return null;
		}

		String text;
		String[] names;
		List<AppNameInfo> result = new ArrayList<AppNameInfo>();
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(in, "UTF-8"), 256);
			br.readLine(); // comment line
			while((text = br.readLine()) != null) {
				names = text.split(",");
				if (names.length < 3) {
					names = new String[] {"error", "error", "error"};
					Log.e("NoTouchLauncher", "loadCache(): the cache file is in a bad format");
				}
				result.add(new AppNameInfo(names[0], names[1], names[2]));
			}
			br.close();
		} catch (IOException e) {
			Log.e("NoTouchLauncher", "loadAppNameInfoList(): Read error", e);
			return null;
		}

		return result;
	}

	public static List<AppNameInfo> loadCache(Context context) {
		return loadAppNameInfoList(context, CACHE_FILE);
	}

	public static List<AppNameInfo> loadExcludeList(Context context, int page) {
		String fileName = EXCLUDE_FILE + "." + String.valueOf(page);
		return loadAppNameInfoList(context, fileName);
	}

	public static List<AppNameInfo> loadRecentList(Context context) {
		return loadAppNameInfoList(context, RECENT_FILE);
	}

	private static void saveAppNameInfoList(Context context, List<AppNameInfo> list, boolean append, String fileName) {
		FileOutputStream out;
		try {
			if (append) {
				out = context.openFileOutput(fileName, Context.MODE_PRIVATE|Context.MODE_APPEND);
			} else {
				out = context.openFileOutput(fileName, Context.MODE_PRIVATE);
			}
		} catch (FileNotFoundException e) {
			return;
		}

		try {
			BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(out, "UTF-8"), 256);
			if (!append) {
				bw.write("# This file is created by NoTouchLauncher.  Please do not edit.\n");
			}

			for (AppNameInfo item : list) {
				bw.write(item.name + "," + item.packageName + "," + item.className);
				bw.write("\n");
			}
			bw.flush();
			bw.close();
		} catch (Exception e) {
			Log.e("NoTouchLauncher", "saveAppNameInfoList(): Write error", e);
			return;
		}
	}

	public static void saveCache(Context context, List<AppNameInfo> list, boolean append) {
		saveAppNameInfoList(context, list, append, CACHE_FILE);
	}

	public static void saveExcludeList(Context context, List<AppNameInfo> list, int page) {
		String fileName = EXCLUDE_FILE + "." + String.valueOf(page);
		saveAppNameInfoList(context, list, false, fileName);
	}

	public static void saveRecentList(Context context, List<AppNameInfo> list) {
		saveAppNameInfoList(context, list, false, RECENT_FILE);
	}

	public static void deleteExcludeList(Context context, int page) {
		String fileName = EXCLUDE_FILE + "." + String.valueOf(page);
		context.deleteFile(fileName);
	}
}
