package jp.deadend.noname.ntl;

import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.content.SharedPreferences;
import android.graphics.drawable.Drawable;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.util.Log;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ExcludeListActivity extends ListActivity {
	static final String KEY_SETNO = "ExcludeListSetNo";
	static final String KEY_SETNAME = "ExcludeListSetName";

	private int mSetNo;
	private boolean[] mEnabled;
	private Map<String, Drawable> mIcons;

	@SuppressWarnings("unchecked")
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		mSetNo = getIntent().getIntExtra(KEY_SETNO, 0);
		String setName = getIntent().getStringExtra(KEY_SETNAME);
		if (setName != null) setTitle(getString(R.string.title_excludelist, setName));

		List<AppNameInfo> disabled = CacheFile.loadExcludeList(getApplicationContext(), mSetNo);

		LoadAppNameListTask task = new LoadAppNameListTask();
		task.execute(disabled);
	}

	@Override
	protected void onPause() {
		List<AppNameInfo> result = new ArrayList<AppNameInfo>();
		ExcludeListAdapter adapter = (ExcludeListAdapter)getListAdapter();
		for (int i=0; i<mEnabled.length; i++) {
			if (mEnabled[i] == false) result.add(adapter.getItem(i));
		}
		CacheFile.saveExcludeList(getApplicationContext(), result, mSetNo);

		super.onPause();
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		CheckBox cb = (CheckBox)v.findViewById(R.id.CheckBox);
		boolean new_val = !cb.isChecked();
		cb.setChecked(new_val);

		mEnabled[position] = new_val;
	}

	private class LoadAppNameListTask extends AsyncTask<List<AppNameInfo>, Void, List<AppNameInfo>> {
		ProgressDialog mProgressDialog;

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(ExcludeListActivity.this, "", getString(R.string.label_loading), true);
		}

		@Override
		protected List<AppNameInfo> doInBackground(List<AppNameInfo> ... disabledList) {
			List<AppNameInfo> result = LoadAppList.load(getApplicationContext());

			mEnabled = new boolean[result.size()];
			for (int i=0; i<result.size(); i++) {
				boolean enabled = true;
				if (disabledList[0] != null) {
					AppNameInfo app = result.get(i);
					for (AppNameInfo disapp : disabledList[0]) {
						if (app.equals(disapp)) enabled = false;
					}
				}
				mEnabled[i] = enabled;
			}

			mIcons = new HashMap<String, Drawable>();
			return result;
		}

		@Override
		protected void onPostExecute(List<AppNameInfo> result) {
			mProgressDialog.dismiss();

			setListAdapter(new ExcludeListAdapter(ExcludeListActivity.this, result));
		}

		@Override
		protected void onCancelled() {
			if (mProgressDialog.isShowing()) {
				mProgressDialog.dismiss();
			}
		}
	}

	private class LoadIconTask extends AsyncTask<String, Void, Drawable> {
		private ImageView mImageView;
		private String mTag;

		public LoadIconTask(ImageView imageView) {
			mImageView = imageView;
			mTag = imageView.getTag().toString();
		}

		@Override
		protected Drawable doInBackground(String... packageName) {
			try {
				Drawable icon = mIcons.get(packageName[0]);
				if (icon == null) {
					icon = getApplicationContext().getPackageManager().getApplicationIcon(packageName[0]);
					mIcons.put(packageName[0], icon);
				}

				return icon;
			} catch (PackageManager.NameNotFoundException e) {
			}

			return null;
		}

		@Override
		protected void onPostExecute(Drawable result) {
			if (mTag.equals(mImageView.getTag())) {
				if (result != null) {
					mImageView.setImageDrawable(result);
				}
			}
		}
	}

	private class ExcludeListAdapter extends ArrayAdapter<AppNameInfo> {
		private LayoutInflater mInflater;

		class ViewHolder {
			ImageView iconView;
			TextView appNameView;
			CheckBox checkBox;
		}

		public ExcludeListAdapter(Context context, List<AppNameInfo> items) {
			super(context, 0, items);
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    	}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			ViewHolder holder;

			View view = convertView;
			if (view == null) {
				view = mInflater.inflate(R.layout.exclude_list_item, null);

				ImageView imv = (ImageView)view.findViewById(R.id.IconView);
				TextView tv = (TextView)view.findViewById(R.id.AppNameView);
				CheckBox cb = (CheckBox)view.findViewById(R.id.CheckBox);

				holder = new ViewHolder();
				holder.iconView = imv;
				holder.appNameView = tv;
				holder.checkBox = cb;

				view.setTag(holder);
    		} else {
				holder = (ViewHolder)view.getTag();
    		}

    		AppNameInfo app = getItem(position);
    		holder.appNameView.setText(app.name);
			holder.checkBox.setChecked(mEnabled[position]);

			holder.iconView.setTag(app.packageName);
			LoadIconTask task = new LoadIconTask(holder.iconView);
			task.execute(app.packageName);

    		return view;
    	}
    }
}
