package jp.deadend.noname.ntl;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.ActivityNotFoundException;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ApplicationInfo;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.animation.AlphaAnimation;
import android.view.animation.Animation;
import android.view.animation.Animation.AnimationListener;
import android.view.KeyEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.Window;
import android.view.WindowManager.LayoutParams;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.util.Log;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class NoTouchLauncher extends Activity {
	private static final long LONG_PRESS_THRESHOLD = 500;
	private EditText mEditText;
	private ListView mListView;
	private List<AppNameInfo> mApps;
	private List<AppNameInfo> mRecentApps;
	private int mNumRecentApps;
	private String mLongPressAction;
	private boolean mCloseAfterLaunch;
	private boolean mCloseAfterLongPress;
	private boolean mVolkeysPage;
	private int mCurrentSet;
	private List<String> mSetTitles;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setSoftInputMode(LayoutParams.SOFT_INPUT_STATE_ALWAYS_HIDDEN);
		setContentView(R.layout.main);

		mCurrentSet = 0;

		mEditText = (EditText)findViewById(R.id.filterEditor);
		mListView = (ListView)findViewById(R.id.mainList);

		mListView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
			@Override
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				launchApp((AppNameInfo)parent.getAdapter().getItem(position));
			}
		});
		mListView.setOnItemLongClickListener(new AdapterView.OnItemLongClickListener() {
			@Override
			public boolean onItemLongClick(AdapterView<?> parent, View view, int position, long id) {
				processLongPress((AppNameInfo)parent.getAdapter().getItem(position));
				return true;
			}
		});
		mListView.setOnKeyListener(new View.OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (event.getAction() == KeyEvent.ACTION_UP) {
					switch (keyCode) {
					case KeyEvent.KEYCODE_DPAD_LEFT:
						int pos = 2*mListView.getFirstVisiblePosition() - mListView.getLastVisiblePosition();
						// getFirstVisiblePosition() - (getLastVisiblePosition() - getFirstVisiblePosition())
						if (pos < 0) {pos = 0;}
						mListView.setSelection(pos);
						return true;
					case KeyEvent.KEYCODE_DPAD_RIGHT:
						mListView.setSelection(mListView.getLastVisiblePosition());
						return true;
					}
				}
				return false;
			}
		});

		findViewById(R.id.buttonReload).setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				LoadAppNameListTask task = new LoadAppNameListTask();
				task.execute();
			}
		});

		findViewById(R.id.buttonPrefs).setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				Intent intent = new Intent(NoTouchLauncher.this,  PrefsActivity.class);
				startActivity(intent);
			}
		});

		findViewById(R.id.buttonSet).setOnClickListener(new OnClickListener() {
			public void onClick(View view) {
				changeSet();
			}
		});

		mEditText.addTextChangedListener(new TextWatcher() {
			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				AppListAdapter adapter = (AppListAdapter)mListView.getAdapter();
				if (adapter != null) adapter.getFilter().filter(s);
			}
			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
			}
			@Override
			public void afterTextChanged(Editable s) {
			}
		});
		mEditText.setOnKeyListener(new View.OnKeyListener() {
			@Override
			public boolean onKey(View v, int keyCode, KeyEvent event) {
				if (event.getAction() == KeyEvent.ACTION_UP) {
					if (keyCode == KeyEvent.KEYCODE_ENTER) {
						ListAdapter adapter = mListView.getAdapter();
						if (adapter.getCount() == 1) {
							if (event.getEventTime() - event.getDownTime() > LONG_PRESS_THRESHOLD) {
								processLongPress((AppNameInfo)adapter.getItem(0));
							} else {
								launchApp((AppNameInfo)adapter.getItem(0));
							}
							return true;
						}
					}
				}
				return false;
			}
		});
	}

	@Override
	public boolean dispatchKeyEvent(KeyEvent event) {
		int keyCode = event.getKeyCode();
		switch (event.getAction()) {
		case KeyEvent.ACTION_UP:
			switch (keyCode) {
			case KeyEvent.KEYCODE_VOLUME_UP:
			case KeyEvent.KEYCODE_VOLUME_DOWN:
				if (mVolkeysPage) {
					if (keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
						int pos = 2*mListView.getFirstVisiblePosition() - mListView.getLastVisiblePosition();
						// getFirstVisiblePosition() - (getLastVisiblePosition() - getFirstVisiblePosition())
						if (pos < 0) {pos = 0;}
						mListView.setSelection(pos);
					} else {
						mListView.setSelection(mListView.getLastVisiblePosition());
					}
				} else {
					int curFontSize = 0;
					AppListAdapter adapter = (AppListAdapter)mListView.getAdapter();
					if (adapter != null) {
						curFontSize = adapter.getFontSize();
						if (keyCode == KeyEvent.KEYCODE_VOLUME_UP) {
							adapter.setFontSize(curFontSize +4);
						} else {
							adapter.setFontSize(curFontSize -4);
						}
						mListView.invalidateViews();
						return true;
					}
				}
				break;
			case KeyEvent.KEYCODE_CAMERA:
			case KeyEvent.KEYCODE_MENU:
			case KeyEvent.KEYCODE_ALT_RIGHT:
				changeSet();
				return true;
			case KeyEvent.KEYCODE_SEARCH:
				if (mRecentApps.size() == 0) return true;
				for (int i=mRecentApps.size() -1; i>0; i--) {
					if (mEditText.getText().toString().equals(mRecentApps.get(i).name)) {
						mEditText.setText(mRecentApps.get(i-1).name);
						return true;
					}
				}
				mEditText.setText(mRecentApps.get(mRecentApps.size()-1).name);
				return true;
			}
			break;
		case KeyEvent.ACTION_DOWN:
			switch (keyCode) {
			case KeyEvent.KEYCODE_VOLUME_UP:
			case KeyEvent.KEYCODE_VOLUME_DOWN:
			case KeyEvent.KEYCODE_ALT_RIGHT:
			case KeyEvent.KEYCODE_SEARCH:
				return true;
			}
		}

		return super.dispatchKeyEvent(event);
	}

	@Override
	protected void onResume() {
		Context context = getApplicationContext();
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(context);
		mCloseAfterLaunch = PrefsActivity.getPrefCloseAfterLaunch(sp);
		mCloseAfterLongPress = PrefsActivity.getPrefCloseAfterLongPress(sp);
		mVolkeysPage = PrefsActivity.getPrefVolkeysPage(sp);
		mLongPressAction = PrefsActivity.getLongPressAction(sp);
		mNumRecentApps = Integer.parseInt(PrefsActivity.getNumRecentApps(sp));
		mSetTitles = PrefsActivity.getSetList(sp);
		if (mSetTitles.size() == 0) mSetTitles.add(getString(R.string.default_set_name));
		if (mCurrentSet >= mSetTitles.size()) mCurrentSet = 0;
		Button btn = (Button)findViewById(R.id.buttonSet);
		btn.setText(mSetTitles.get(mCurrentSet));
		mRecentApps = CacheFile.loadRecentList(context);
		if (mRecentApps != null) {
			while (mRecentApps.size() > mNumRecentApps) {
				mRecentApps.remove(0);
			}
		} else {
			mRecentApps = new ArrayList<AppNameInfo>();
		}

		if (mApps != null) {
			displayList();
		} else {
			try {
				mApps = CacheFile.loadCache(context);
				if (mApps != null) {
					Collections.sort(mApps);
					displayList();
				} else {
					LoadAppNameListTask task = new LoadAppNameListTask();
					task.execute();
				}
			} catch (Exception e) {
				Log.e("NoTouchLauncher", "Error: ", e);
			}
		}

		super.onResume();
	}

	private void changeSet() {
		final Button btn = (Button)findViewById(R.id.buttonSet);
		mCurrentSet++;
		if (mCurrentSet >= mSetTitles.size()) mCurrentSet = 0;

		AlphaAnimation anim = new AlphaAnimation(1.0f, 0.0f);
		anim.setDuration(150);
		anim.setAnimationListener(new AnimationListener() {
			@Override
			public void onAnimationEnd(final Animation animation) {
				displayList();
				btn.setText(mSetTitles.get(mCurrentSet));
			}
			@Override
			public void onAnimationRepeat(final Animation animation) {}
			@Override
			public void onAnimationStart(final Animation animation) {}
		});
		mListView.startAnimation(anim);
	}

	private void displayList() {
		int fontSize = 0;
		AppListAdapter oldAdapter = (AppListAdapter)mListView.getAdapter();
		if (oldAdapter != null) {
			fontSize = oldAdapter.getFontSize();
		} else {
			fontSize = PrefsActivity.getFontSize(PreferenceManager.getDefaultSharedPreferences(getApplicationContext()));
		}

		List<AppNameInfo> listToDisp = new ArrayList<AppNameInfo>(mApps);
		for (AppNameInfo app : mRecentApps) {
			if (listToDisp.remove(app)) listToDisp.add(0, app);
		}
		List<AppNameInfo> excludeList = CacheFile.loadExcludeList(getApplicationContext(), mCurrentSet);
		if (excludeList != null) {
			for (AppNameInfo e_app : excludeList) {
				listToDisp.remove(e_app);
			}
		}

		AppListAdapter adapter = new AppListAdapter(NoTouchLauncher.this, listToDisp);
		if (fontSize != 0) {
			adapter.setFontSize(fontSize);
		}
		mListView.setAdapter(adapter);
	}

	@Override
	protected void onPause() {
		Context context = getApplicationContext();
		CacheFile.saveRecentList(context, mRecentApps);

		int curFontSize = 0;
		AppListAdapter adapter = (AppListAdapter)mListView.getAdapter();
		if (adapter != null) curFontSize = adapter.getFontSize();
		if (curFontSize != 0) {
			PreferenceManager.getDefaultSharedPreferences(context)
				.edit()
				.putInt(PrefsActivity.KEY_FONTSIZE, curFontSize)
				.commit();
		}

		super.onPause();
	}

	private void launchApp(AppNameInfo app) {
		Intent intent = new Intent();
		intent.setAction(Intent.ACTION_MAIN);
		intent.addCategory(Intent.CATEGORY_LAUNCHER);
		intent.setClassName(app.packageName, app.className);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_RESET_TASK_IF_NEEDED);

		mRecentApps.remove(app);
		mRecentApps.add(app);
		while (mRecentApps.size() > mNumRecentApps) {
			mRecentApps.remove(0);
		}

		try {
			startActivity(intent);
			if (mCloseAfterLaunch) finish();
		} catch (ActivityNotFoundException e) {
			Toast.makeText(this, R.string.label_not_found, Toast.LENGTH_LONG).show();
		}
		mEditText.setText("");
	}

	public void processLongPress(AppNameInfo app) {
		if (mLongPressAction.equals(PrefsActivity.LONGPRESS_APP_DETAIL)) {
			showAppDetails(app);
		} else if (mLongPressAction.equals(PrefsActivity.LONGPRESS_MARKET)) {
			openMarket(app);
		} else if (mLongPressAction.equals(PrefsActivity.LONGPRESS_CHOOSE)) {
			final AppNameInfo tmp_app = app;
			final String[] items = getResources().getStringArray(R.array.label_longpress_dialog);
			new AlertDialog.Builder(this)
				.setTitle(R.string.pref_longpress_dialog_title)
				.setItems(items, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int item) {
						if (items[item].equals(getString(R.string.label_longpress_detail))) {
							showAppDetails(tmp_app);
						} else if (items[item].equals(getString(R.string.label_longpress_gp))) {
							openMarket(tmp_app);
						}
					}
				})
				.show();
		}
		mEditText.setText("");
	}

	public void showAppDetails(AppNameInfo app) {
		try {
			PackageManager packageManager = getApplicationContext().getPackageManager();
			ApplicationInfo applicationInfo = packageManager.getApplicationInfo(app.packageName, PackageManager.GET_META_DATA);
		} catch (PackageManager.NameNotFoundException exception) {
			Toast.makeText(this, R.string.label_not_found, Toast.LENGTH_LONG).show();
			return;
		}

		Intent intent = new Intent();
		final int apiLevel = Build.VERSION.SDK_INT;
		if (apiLevel >= 9) {
			intent.setAction(android.provider.Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
			Uri uri = Uri.fromParts("package", app.packageName, null);
			intent.setData(uri);
		} else {
			final String appPkgName = (apiLevel == 8 ? "pkg" : "com.android.settings.ApplicationPkgName");
			intent.setAction(Intent.ACTION_VIEW);
			intent.setClassName("com.android.settings", "com.android.settings.InstalledAppDetails");
			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
			intent.putExtra(appPkgName, app.packageName);
		}

		startActivity(intent);
		if (mCloseAfterLongPress) finish();
	}

	public void openMarket(AppNameInfo app) {
		Uri uri = Uri.parse("market://details?id=" + app.packageName);
		Intent intent = new Intent(Intent.ACTION_VIEW, uri);
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		startActivity(intent);
		if (mCloseAfterLongPress) finish();
	}

	private class LoadAppNameListTask extends AsyncTask<Void, Void, List<AppNameInfo>> {
		ProgressDialog mProgressDialog;

		@Override
		protected void onPreExecute() {
			mProgressDialog = ProgressDialog.show(NoTouchLauncher.this, "", getString(R.string.label_loading), true);
		}

		@Override
		protected List<AppNameInfo> doInBackground(Void ... params) {
			List<AppNameInfo> result = LoadAppList.load(getApplicationContext());
			return result;
		}

		@Override
		protected void onPostExecute(List<AppNameInfo> result) {
			mProgressDialog.dismiss();

			mApps = result;
			displayList();
			mEditText.setText("");
		}

		@Override
		protected void onCancelled() {
			if (mProgressDialog.isShowing()) {
				mProgressDialog.dismiss();
			}
		}
	}
}
