package jp.deadend.noname.ntl;

import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceManager;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.preference.Preference.OnPreferenceClickListener;
import android.view.Window;
import android.view.View;
import android.util.Log;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class PrefsActivity extends PreferenceActivity implements OnPreferenceChangeListener {
	private static final String KEY_CLOSE_AFTER_LAUNCH = "CloseAfterLaunch";
	private static final String KEY_CLOSE_AFTER_LONG_PRESS = "CloseAfterLongPress";
	private static final String KEY_VOLKEYS_PAGE = "VolumeKeysPage";
	private static final String KEY_LONGPRESS = "LongPress";
	private static final String KEY_NUM_RECENTAPPS = "NumberOfRecentApps";
	static final String KEY_FONTSIZE = "FontSize";
	static final String KEY_SET_LIST = "SetList";
	static final String LONGPRESS_APP_DETAIL = "App detail";
	static final String LONGPRESS_MARKET = "Market";
	static final String LONGPRESS_CHOOSE = "Choose";
	static final String DEFAULT_LONGPRESS = LONGPRESS_APP_DETAIL;
	static final int DEFAULT_FONT_SIZE = 24;
	private static final String DEFAULT_NUM_RECENTAPPS = "5";
	private static final String[] NUM_RECENTAPPS_LIST = {"0", "5", "10"};
	private static final String[] LONGPRESS_LIST = {LONGPRESS_APP_DETAIL, LONGPRESS_MARKET, LONGPRESS_CHOOSE};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		super.onCreate(savedInstanceState);

		PreferenceManager pm = getPreferenceManager();
		PreferenceScreen ps = pm.createPreferenceScreen(this);
		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(this);

		// 起動後に閉じる
		CheckBoxPreference checkBoxPr = new CheckBoxPreference(this);
		checkBoxPr.setKey(KEY_CLOSE_AFTER_LAUNCH);
		checkBoxPr.setTitle(R.string.pref_close_after_launch_title);
		checkBoxPr.setChecked(sp.getBoolean(KEY_CLOSE_AFTER_LAUNCH, true));
		ps.addPreference(checkBoxPr);

		// 長押し後に閉じる
		checkBoxPr = new CheckBoxPreference(this);
		checkBoxPr.setKey(KEY_CLOSE_AFTER_LONG_PRESS);
		checkBoxPr.setTitle(R.string.pref_close_after_longpress_title);
		checkBoxPr.setChecked(sp.getBoolean(KEY_CLOSE_AFTER_LONG_PRESS, false));
		ps.addPreference(checkBoxPr);

		// ボリュームキー
		checkBoxPr = new CheckBoxPreference(this);
		checkBoxPr.setKey(KEY_VOLKEYS_PAGE);
		checkBoxPr.setTitle(R.string.pref_volkeys_page_title);
		checkBoxPr.setChecked(sp.getBoolean(KEY_VOLKEYS_PAGE, false));
		ps.addPreference(checkBoxPr);

		// 履歴サイズ
		ListPreference listPr = new ListPreference(this);
		listPr.setKey(KEY_NUM_RECENTAPPS);
		listPr.setSummary(sp.getString(listPr.getKey(), DEFAULT_NUM_RECENTAPPS));
		listPr.setTitle(R.string.pref_history_size_title);
		listPr.setDialogTitle(R.string.pref_history_size_dialog_title);
		listPr.setEntries(NUM_RECENTAPPS_LIST);
		listPr.setEntryValues(NUM_RECENTAPPS_LIST);
		listPr.setOnPreferenceChangeListener(this);
		ps.addPreference(listPr);

		// 長押し動作
		listPr = new ListPreference(this);
		listPr.setKey(KEY_LONGPRESS);
		listPr.setTitle(R.string.pref_longpress_title);
		listPr.setDialogTitle(R.string.pref_longpress_dialog_title);
		String[] entries = getResources().getStringArray(R.array.label_longpress_pref);
		listPr.setEntries(entries);
		listPr.setEntryValues(LONGPRESS_LIST);
		listPr.setOnPreferenceChangeListener(this);
		ps.addPreference(listPr);
		listPr.setSummary(listPr.getEntry());

		// 非表示アプリ設定画面へ
		Preference pr = new Preference(this);
		pr.setTitle(R.string.pref_hide_title);
		pr.setOnPreferenceClickListener(new OnPreferenceClickListener() {
			@Override
			public boolean onPreferenceClick(Preference preference) {
				Intent intent = new Intent(PrefsActivity.this, SetListActivity.class);
				startActivity(intent);
				return true;
			}
		});
		ps.addPreference(pr);

		setPreferenceScreen(ps);
	}

	@Override
	public boolean onPreferenceChange(Preference preference, Object newValue) {
		if(newValue != null){
			if (preference instanceof ListPreference) {
				int listId = ((ListPreference)preference).findIndexOfValue((String)newValue);
				CharSequence[] entries = ((ListPreference)preference).getEntries();
				preference.setSummary(entries[listId]);
			} else {
				preference.setSummary((CharSequence)newValue);
			}
			return true;
		}

		return false;
	}

	static boolean getPrefCloseAfterLaunch(SharedPreferences sp) {
		return sp.getBoolean(KEY_CLOSE_AFTER_LAUNCH, true);
	}

	static boolean getPrefCloseAfterLongPress(SharedPreferences sp) {
		return sp.getBoolean(KEY_CLOSE_AFTER_LONG_PRESS, false);
	}

	static boolean getPrefVolkeysPage(SharedPreferences sp) {
		return sp.getBoolean(KEY_VOLKEYS_PAGE, false);
	}

	static int getFontSize(SharedPreferences sp) {
		int val;
		try {
			val = sp.getInt(KEY_FONTSIZE, DEFAULT_FONT_SIZE);
		} catch (ClassCastException e) {
			sp.edit()
				.remove(KEY_FONTSIZE)
				.commit();
			return DEFAULT_FONT_SIZE;
		}

		return val;
	}

	static String getLongPressAction(SharedPreferences sp) {
		return sp.getString(KEY_LONGPRESS, DEFAULT_LONGPRESS);
	}

	static String getNumRecentApps(SharedPreferences sp) {
		return sp.getString(KEY_NUM_RECENTAPPS, DEFAULT_NUM_RECENTAPPS);
	}

	static List<String> getSetList(SharedPreferences sp) {
		List<String> result;
		String str = sp.getString(KEY_SET_LIST, "");
		if (str.length() > 0) {
			result = new ArrayList<String>(Arrays.asList(str.split("\\|")));
		} else {
			result = new ArrayList<String>();
		}

		return result;
	}
}
