package jp.deadend.noname.ntl;

import java.util.HashMap;
import java.util.Locale;

class RomajiConverter {
	// ローマ字辞書
	private HashMap<String, String> mRomajiMap = new HashMap<String, String>();
	private HashMap<String, String> mMaybeRomajiMap = new HashMap<String, String>();

	public RomajiConverter() {
		HashMap<String, String> m = mRomajiMap;
		m.put("a", "あ");m.put("i", "い");m.put("u", "う");m.put("e", "え");m.put("o", "お");
		m.put("ka", "か");m.put("ki", "き");m.put("ku", "く");m.put("ke", "け");m.put("ko", "こ");
		m.put("sa", "さ");m.put("si", "し");m.put("su", "す");m.put("se", "せ");m.put("so", "そ");
		m.put("ta", "た");m.put("ti", "ち");m.put("tu", "つ");m.put("tsu", "つ");m.put("te", "て");m.put("to", "と");
		m.put("na", "な");m.put("ni", "に");m.put("nu", "ぬ");m.put("ne", "ね");m.put("no", "の");
		m.put("ha", "は");m.put("hi", "ひ");m.put("hu", "ふ");m.put("he", "へ");m.put("ho", "ほ");
		m.put("ma", "ま");m.put("mi", "み");m.put("mu", "む");m.put("me", "め");m.put("mo", "も");
		m.put("ya", "や");m.put("yu", "ゆ");m.put("yo", "よ");
		m.put("ra", "ら");m.put("ri", "り");m.put("ru", "る");m.put("re", "れ");m.put("ro", "ろ");
		m.put("wa", "わ");m.put("wi", "うぃ");m.put("we", "うぇ");m.put("wo", "を");m.put("nn", "ん");
		m.put("ga", "が");m.put("gi", "ぎ");m.put("gu", "ぐ");m.put("ge", "げ");m.put("go", "ご");
		m.put("za", "ざ");m.put("zi", "じ");m.put("zu", "ず");m.put("ze", "ぜ");m.put("zo", "ぞ");
		m.put("da", "だ");m.put("di", "ぢ");m.put("du", "づ");m.put("de", "で");m.put("do", "ど");
		m.put("ba", "ば");m.put("bi", "び");m.put("bu", "ぶ");m.put("be", "べ");m.put("bo", "ぼ");
		m.put("pa", "ぱ");m.put("pi", "ぴ");m.put("pu", "ぷ");m.put("pe", "ぺ");m.put("po", "ぽ");
		m.put("va", "う゛ぁ");m.put("vi", "う゛ぃ");m.put("vu", "う゛");m.put("ve", "う゛ぇ");m.put("vo", "う゛ぉ");

		m.put("xa", "ぁ");m.put("xi", "ぃ");m.put("xu", "ぅ");m.put("xe", "ぇ");m.put("xo", "ぉ");
		m.put("xtu", "っ");m.put("xke", "ヶ");
		m.put("la", "ぁ");m.put("li", "ぃ");m.put("lu", "ぅ");m.put("le", "ぇ");m.put("lo", "ぉ");
		m.put("ltu", "っ");m.put("lke", "ヶ");

		m.put("xya", "ゃ");m.put("xyu", "ゅ");m.put("xyo", "ょ");
		m.put("lya", "ゃ");m.put("lyu", "ゅ");m.put("lyo", "ょ");
		m.put("fa", "ふぁ");m.put("fi", "ふぃ");m.put("fu", "ふ");m.put("fe", "ふぇ");m.put("fo", "ふぉ");
		m.put("kya", "きゃ");m.put("kyu", "きゅ");m.put("kyo", "きょ");
		m.put("gya", "ぎゃ");m.put("gyu", "ぎゅ");m.put("gyo", "ぎょ");
		m.put("sya", "しゃ");m.put("syu", "しゅ");m.put("syo", "しょ");
		m.put("sha", "しゃ");m.put("shi", "し");m.put("shu", "しゅ");m.put("she", "しぇ");m.put("sho", "しょ");
		m.put("ja",  "じゃ");m.put("ji",  "じ");m.put("ju", "じゅ");m.put("je", "じぇ");m.put("jo", "じょ");
		m.put("cha", "ちゃ");m.put("chi", "ち");m.put("chu", "ちゅ");m.put("che", "ちぇ");m.put("cho", "ちょ");
		m.put("tya", "ちゃ");m.put("tyu", "ちゅ");m.put("tye", "ちぇ");m.put("tyo", "ちょ");
		m.put("dha", "でゃ");m.put("dhi", "でぃ");m.put("dhu", "でゅ");m.put("dhe", "でぇ");m.put("dho", "でょ");
		m.put("dya", "ぢゃ");m.put("dyi", "ぢぃ");m.put("dyu", "ぢゅ");m.put("dye", "ぢぇ");m.put("dyo", "ぢょ");
		m.put("nya", "にゃ");m.put("nyu", "にゅ");m.put("nyo", "にょ");
		m.put("hya", "ひゃ");m.put("hyu", "ひゅ");m.put("hyo", "ひょ");
		m.put("pya", "ぴゃ");m.put("pyu", "ぴゅ");m.put("pyo", "ぴょ");
		m.put("bya", "びゃ");m.put("byu", "びゅ");m.put("byo", "びょ");
		m.put("mya", "みゃ");m.put("myu", "みゅ");m.put("myo", "みょ");
		m.put("rya", "りゃ");m.put("ryu", "りゅ");m.put("rye", "りぇ");m.put("ryo", "りょ");

		m = mMaybeRomajiMap;
		m.put("k", "[かきくけこっ]");
		m.put("s", "[さしすせそっ]");
		m.put("t", "[たちつてとっ]");
		m.put("ts", "つ");
		m.put("n", "[なにぬねのん]");
		m.put("h", "[はひふへほっ]");
		m.put("m", "[まみむめも]");
		m.put("y", "[やゆよ]");
		m.put("r", "[らりるれろっ]");
		m.put("w", "[わうを]");
		m.put("g", "[がぎぐげごっ]");
		m.put("z", "[ざじずぜぞっ]");
		m.put("d", "[だぢづでどっ]");
		m.put("b", "[ばびぶべぼっ]");
		m.put("p", "[ぱぴぷぺぽっ]");
		m.put("v", "う゛");
		m.put("c", "ち");

		m.put("x", "[ぁぃぅぇぉゃゅょっヶ]");
		m.put("xt", "っ");m.put("xk", "ヶ");
		m.put("l", "[ぁぃぅぇぉゃゅょっヶ]");
		m.put("lt", "っ");m.put("lk", "ヶ");

		m.put("xy", "[ゃゅょ]");
		m.put("ly", "[ゃゅょ]");
		m.put("f", "ふ");
		m.put("ky", "き[ゃゅょ]");
		m.put("gy", "ぎ[ゃゅょ]");
		m.put("sy", "し[ゃゅょ]");
		m.put("sh", "し");
		m.put("j",  "じ");
		m.put("ch", "ち");
		m.put("ty", "ち[ゃゅぇょ]");
		m.put("dh", "で[ゃぃゅぇょ]");
		m.put("dy", "ぢ[ゃぃゅぇょ]");
		m.put("ny", "に[ゃゅょ]");
		m.put("hy", "ひ[ゃゅょ]");
		m.put("py", "ぴ[ゃゅょ]");
		m.put("by", "び[ゃゅょ]");
		m.put("my", "み[ゃゅょ]");
		m.put("ry", "り[ゃゅぇょ]");
	}

	private String romajiToHiragana(String str) {
		int pos = 0;
		final int len = str.length();
		for (int i=0; i<len; i++) {
			if (!isAlphabet(str.charAt(i))) return str;
		}

		String roma = str.toLowerCase(Locale.US);
		pos = 0;
		StringBuilder result = new StringBuilder();
		int end_ofs = 1;
		String one_kana = null;
		char here, next;
		while (pos < len) {
			here = roma.charAt(pos);
			// 音引きの処理
			if (here == '-') {
				result.append("ー");
				pos++;
				continue;
			}
			// 最後の文字
			if (pos == len-1) {
				one_kana = mRomajiMap.get(Character.toString(here));
				if (one_kana != null) {
					result.append(one_kana);
				} else {
					one_kana = mMaybeRomajiMap.get(new String(new char[] {here}));
					if (one_kana != null) {
						result.append(one_kana);
					} else {
						result.append(here);
					}
				}
				break;
			}
			next = roma.charAt(pos+1);
			// 「ん」の処理
			if (here == 'n') {
				if (!isVowel(next) && next != 'n' && next != 'y') {
					result.append("ん");
					pos++;
					continue;
				}
			}
			// 「っ」の処理
			if (!isVowel(here) && here == next) {
				result.append("っ");
				pos += 1;
				continue;
			}
			// それ以外
			while (one_kana == null) {
				if (pos+end_ofs > len) {
					one_kana = mMaybeRomajiMap.get(roma.substring(pos));
					if (one_kana != null) {
						result.append(one_kana);
					} else {
						result.append(roma.substring(pos));
					}
					return result.toString();
				}
				one_kana = mRomajiMap.get(roma.substring(pos, pos+end_ofs));
				end_ofs++;
			}
			result.append(one_kana);
			pos = pos+end_ofs-1;
			end_ofs = 1;
			one_kana = null;
		}

		return result.toString();
	}

	private static boolean isAlphabet(char code) {
		return ((code == 0x2d) || (code >= 0x41 && code <= 0x5A) || (code >= 0x61 && code <= 0x7A)) ? true : false;
	}

	private static boolean isVowel(char p) {
		switch (p) {
		case 'a':
		case 'i':
		case 'u':
		case 'e':
		case 'o':
			return true;
		default:
			return false;
		}
	}

	/**
	* 文字列・改
	*
	* @author 佐藤 雅俊さん <okome@siisise.net> http://siisise.net/java/lang/ のコードを改変
	*/
	/**
	* ひらがなを全角カタカナにする
	*/
	private static String hiraganaToKatakana(String str) {
		if (str == null) return null;

		StringBuilder str2 = new StringBuilder();
		for (int i=0; i<str.length(); i++) {
			char ch = str.charAt(i);

			if (ch >= 0x3040 && ch <= 0x309A) {
				ch += 0x60;
			}
			str2.append(ch);
		}

		int idx = str2.indexOf("ウ゛");
		if (idx != -1) str2.replace(idx, idx+2, "ヴ");

		return str2.toString();
	}

	// 半角アルファベットと記号を全角に変換，その他の字はそのまま
	private static char hankakuToZenkaku(char code) {
		if (code == 0x20) // スペースだけ、特別
			return 0x3000;
		if (code >= 0x21 && code <= 0x7E)
			return (char)(code - 0x20 + 0xFF00);

		return code;
	}

	public String makeRegExp(String str) {
		String hiragana = romajiToHiragana(str);
		String katakana = hiraganaToKatakana(hiragana);
		StringBuilder zenkaku = new StringBuilder();
		for (int i=0; i<str.length(); i++) {
			zenkaku.append(hankakuToZenkaku(str.charAt(i)));
		}

		StringBuilder result = new StringBuilder(str);
		if (!str.equals(hiragana) && !hiragana.equals("")) {
			result.append("|");
			result.append(hiragana);
		}
		if (!str.equals(katakana) && !katakana.equals("")) {
			result.append("|");
			result.append(katakana);
		}
		if (!str.equals(zenkaku.toString())) {
			result.append("|");
			result.append(zenkaku);
		}

		return result.toString();
	}

}
