package jp.deadend.noname.ntl;

import android.app.AlertDialog;
import android.app.Dialog;
import android.app.ListActivity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.view.Window;
import android.widget.AdapterView.AdapterContextMenuInfo;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.util.Log;

import java.util.List;

public class SetListActivity extends ListActivity {
	private static final int DIALOG_RENAME = 0;
	private static final int DIALOG_DELETE = 1;
	private static final int MENU_RENAME = 0;
	private static final int MENU_DELETE = 1;
	private ArrayAdapter<String> mAdapter;
	private List<String> mList;
	private int mItemIndex;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);

		mItemIndex = 0;

		SharedPreferences sp = PreferenceManager.getDefaultSharedPreferences(getApplicationContext());
		mList = PrefsActivity.getSetList(sp);
		if (mList.size() == 0) {
			mList.add(getString(R.string.default_set_name));
		}
		mList.add(0, getString(R.string.pref_setlist_add));

		mAdapter = new SetListAdapter(this, mList);
		setListAdapter(mAdapter);

		registerForContextMenu(getListView());
	}

	@Override
	protected void onListItemClick(ListView l, View v, int position, long id) {
		super.onListItemClick(l, v, position, id);

		if (position == 0) {
			addNewItem();
		} else {
			Intent intent = new Intent(this, ExcludeListActivity.class);
			intent.putExtra(ExcludeListActivity.KEY_SETNO, position-1);
			intent.putExtra(ExcludeListActivity.KEY_SETNAME, mList.get(position));
			startActivity(intent);
 		}
	}

	@Override
	protected void onPause() {
		StringBuilder sets = new StringBuilder();
		for (int i=1; i<mList.size(); i++) {
			sets.append(mList.get(i));
			sets.append('|');
		}

		PreferenceManager.getDefaultSharedPreferences(getApplicationContext())
			.edit()
			.putString(PrefsActivity.KEY_SET_LIST, sets.toString())
			.commit();

		super.onPause();
	}

	private void addNewItem() {
		int set=1;
		while (mList.contains("new set " + String.valueOf(set))) {
			set++;
		}

		mList.add("new set " + String.valueOf(set));
		mAdapter.notifyDataSetChanged();
	}

	private boolean contains(String s) {
		for (String listStr : mList) {
			if (listStr.equalsIgnoreCase(s)) {
				return true;
			}
		}

		return false;
	}

	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo)menuInfo;
		if (info.position == 0) return;

		super.onCreateContextMenu(menu, v, menuInfo);

		menu.setHeaderTitle(mList.get(info.position));
		menu.add(0, MENU_RENAME, 0, R.string.pref_setlist_rename);
		menu.add(0, MENU_DELETE, 0, R.string.pref_setlist_delete);
	}

	public boolean onContextItemSelected(MenuItem item) {
		AdapterContextMenuInfo info = (AdapterContextMenuInfo) item.getMenuInfo();
		mItemIndex = info.position;

		switch (item.getItemId()) {
		case MENU_RENAME:
			removeDialog(DIALOG_RENAME);
			showDialog(DIALOG_RENAME);
			return true;
		case MENU_DELETE:
			removeDialog(DIALOG_DELETE);
			showDialog(DIALOG_DELETE);
			return true;
		default:
			return super.onContextItemSelected(item);
		}
	}

	@Override
	protected Dialog onCreateDialog(int id) {
		Dialog dialog;
		switch(id) {
		case DIALOG_RENAME:
			final EditText edtInput = new EditText(this);
			edtInput.setSingleLine();
			edtInput.setText(mList.get(mItemIndex));
			dialog = new AlertDialog.Builder(this)
				.setTitle(R.string.pref_setlist_rename)
				.setView(edtInput)
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						String str = edtInput.getText().toString();
						if (str.length() > 0 && !contains(str)) {
							mList.set(mItemIndex, str);
							mAdapter.notifyDataSetChanged();
						}
					}
				})
				.setNegativeButton(R.string.label_CANCEL, null)
				.setCancelable(true)
				.create();
			break;
		case DIALOG_DELETE:
			dialog = new AlertDialog.Builder(this)
				.setTitle(R.string.pref_setlist_delete)
				.setMessage(getString(R.string.label_delete_item, mList.get(mItemIndex)))
				.setPositiveButton(R.string.label_OK, new DialogInterface.OnClickListener() {
					public void onClick(DialogInterface dialog, int whichButton) {
						mList.remove(mItemIndex);
						mAdapter.notifyDataSetChanged();
						CacheFile.deleteExcludeList(getApplicationContext(), mItemIndex-1);
					}
				})
				.setNegativeButton(R.string.label_CANCEL, null)
				.setCancelable(true)
				.create();
			break;
		default:
			dialog = null;
		}

		return dialog;
	}

	private static class SetListAdapter extends ArrayAdapter<String> {
		private LayoutInflater mInflater;

		SetListAdapter(Context context, List<String> objects) {
			super(context, 0, objects);
			mInflater = (LayoutInflater)context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
		}

		@Override
		public View getView(int position, View convertView, ViewGroup parent) {
			TextView tv = (TextView)convertView;
			if (tv == null) {
				tv = (TextView)mInflater.inflate(android.R.layout.simple_list_item_1, null);
			}

			if (position != 0) {
				tv.setText("\"" + getItem(position) + "\"");
			} else {
				tv.setText(getItem(position));
			}

			return tv;
		}
	}
}
